;;
;; editing 'onset' for individually selected chords inside a chord-seq, by
;; typing in the number box or using vertical drag.  Feature request from perti
;; june 26th.
;;
;; Anders Vinjar, june 26th 2019
;; 

(defmethod GET-slot-LIST ((self omcontrols-view)) 
  '(("midic" midic) ("channel" chan) ("dur" dur) ("dyn" dyn) ("port" port) ("offset" offset) ("onset" onset)))

(defmethod update-slot-edit ((self scorePanel))
  (unless (score-page-mode self)
    (let ((control (slotedit (ctr-view (om-view-container self))))
          (slotmode (slots-mode self))
          (firstnote nil))
      (loop for obj in (reverse (selection? self)) while (not firstnote) do
            (setq firstnote (get-first-note obj)))
      (enable-numbox control nil)
      (when firstnote ; (and (string-equal (obj-mode self) "note") firstnote)
        (enable-numbox control t)
        (set-value control nil)
        (cond
         ((equal slotmode 'dur)
          (setf (min-val control) 0)
          (setf (max-val control) 100000)
          (set-value control (extent->ms firstnote))
          (setf (afterfun control) 
                #'(lambda (x)  
                    (loop for item in (selection? self) do
                          (set-dur item (value x)))
                    (update-panel self))))
        ((equal slotmode 'port)
         (setf (min-val control) 0)
         (setf (max-val control) 255)
         (set-value control (port firstnote))
         (setf (afterfun control) 
               #'(lambda (x)  
                   (loop for item in (selection? self) do
                         (set-port item (value x)))
                   (update-panel self))))
        
        ((equal slotmode 'offset)
         (setf (min-val control) -100000)
         (setf (max-val control) 100000)
         (set-value control (offset->ms firstnote))
         (setf (afterfun control) 
               #'(lambda (x)  
                   (loop for item in (selection? self) do
                         (set-offset-ms item (value x)))
                   (update-panel self))))
        ((equal slotmode 'onset)
	 (enable-numbox control (if (string-equal (obj-mode self) "chord") t nil))
         (setf (min-val control) 0)
         (setf (max-val control) (+ (offset->ms firstnote self) 100000))
	 (set-value control (offset->ms firstnote self))
	 (setf (afterfun control)
	       (progn
		 #'(lambda (x)
		     (loop
			for item in (selection? self)
			do (set-chords-offset self item (value x)))
		     (update-panel self)))))
	((equal slotmode 'dyn)
         (setf (min-val control) 0)
         (setf (max-val control) 127)
         (set-value control (vel firstnote))
         (setf (afterfun control)
               #'(lambda (x) 
                   (loop for item in (selection? self) do
                         (set-vel item (value x)))
                   (update-panel self))))
        ((equal slotmode 'chan)
         (setf (min-val control) 1)
         (setf (max-val control) 16)
         (set-value control (chan firstnote))
         (setf (afterfun control)
               #'(lambda (x) 
                   (loop for item in (selection? self) do
                         (set-channel item (value x)))
                   (update-panel self))))
        ((equal slotmode 'midic)
         (setf (min-val control) 0)
         (setf (max-val control) 12700)
         (set-value control (midic firstnote))
         (setf (afterfun control) 
               #'(lambda (x) 
                   (loop for item in (selection? self) do
                         (when (note-p item)
                           (change-midic item (value x))))
                   (update-panel self))))
        ((equal slotmode 'chord-offset)
         (setf (min-val control) 0)
         (setf (max-val control) 1000000)
         (set-value control (offset->ms (object (om-view-container self))))
         (setf (afterfun control) 
               #'(lambda (x) 
                   (all-chords-2-ms (parent (object (om-view-container self))))
                   (setf (offset (object (om-view-container self))) (max 0 (value x))) 
                   (normalize-chords-x (parent (object (om-view-container self))))
                   (update-panel self t))))
        ((equal slotmode 'non)
         (setf (min-val control) 0)
         (setf (max-val control) 0)
         (setf (afterfun control) 
               #'(lambda (x) (declare (ignore x)) (om-beep)))))
             )
     (om-invalidate-view self t))))
